@@ -5,13 +5,18 @@
  * Copyright (C) 2004-2006  Nathan J. Conrad <t98502@users.sourceforge.net>
  * Copyright (C) 2004       Karl Soderstrom <ks@xanadunet.net>
  *
- * parts of the RFC822 timezone decoding were taken from the gmime 
- * source written by 
+ * parts of the RFC822 timezone decoding were reused from the gmime source
  *
- * Authors: Michael Zucchi <notzed@helixcode.com>
- *          Jeffrey Stedfast <fejj@helixcode.com>
+ *    Authors: Michael Zucchi <notzed@helixcode.com>
+ *             Jeffrey Stedfast <fejj@helixcode.com>
  *
- * Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ *    Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ * 
+ * The date formatting was reused from the Evolution code base
+ *
+ *    Author: Chris Lahey <clahey@ximian.com
+ *
+ *    Copyright 2001, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,90 +44,32 @@
 #include <libxml/parser.h>
 #include <libxml/entities.h>
 #include <libxml/HTMLparser.h>
+#include <libxml/xpath.h>
+
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <pango/pango-types.h>
 #include <gtk/gtk.h>
 
 #include <sys/stat.h>
+#include <errno.h>
 #include <string.h>
 #include <locale.h>
 #include <time.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <unistd.h>	/* fsync */
+
 #include "support.h"
 #include "common.h"
 #include "conf.h"
-#include "support.h"
 #include "feed.h"
+#include "support.h"
 #include "debug.h"
 
-static gchar *standard_encoding = { "UTF-8" };
-
 static gchar *lifereaUserPath = NULL;
 
-gchar * convertCharSet(gchar * from_encoding, gchar * to_encoding, gchar * string);
-
-void addToHTMLBufferFast(gchar **buffer, const gchar *string) {
-	
-	if(NULL == string)
-		return;
-	
-	if(NULL != *buffer) {
-		gulong oldlength = strlen(*buffer);
-		gulong newlength = strlen(string);
-		gulong allocsize = (((oldlength+newlength+1L)/512L)+1L)*512L; /* Round up to nearest 512 KB */
-		*buffer = g_realloc(*buffer, allocsize);
-		g_memmove(&((*buffer)[oldlength]), string, newlength+1L );
-	} else {
-		*buffer = g_strdup(string);
-	}
-}
-
-void addToHTMLBuffer(gchar **buffer, const gchar *string) {
-	
-	if(NULL == string)
-		return;
-	
-	if(NULL != *buffer) {
-		gulong oldlength = strlen(*buffer);
-		gulong newlength = strlen(string);
-		gulong allocsize = (oldlength+newlength+1L);
-		*buffer = g_realloc(*buffer, allocsize);
-		g_memmove(&((*buffer)[oldlength]), string, newlength+1L );
-	} else {
-		*buffer = g_strdup(string);
-	}
-}
-
-/* converts the string string encoded in from_encoding (which
-   can be NULL) to to_encoding, frees the original string and 
-   returns the result */
-gchar * convertCharSet(gchar * from_encoding, gchar * to_encoding, gchar * string) {
-	gsize	bw, br;
-	gchar	*new = NULL;
-	GError *err = NULL;
-
-	if(NULL == from_encoding)
-		from_encoding = standard_encoding;
-		
-	if(NULL != string) {		
-		new = g_convert(string, strlen(string), to_encoding, from_encoding, &br, &bw, &err);
-		if (err != NULL) {
-			g_warning("error converting character set: %s\n", err->message);
-			g_error_free (err);
-		}
-		if(NULL != new)
-			g_free(string);
-		else
-			new = string;
-	} else {	
-		return g_strdup("");
-	}
-
-	return new;
-}
-
-gchar * convertToHTML(gchar * string) { return string; } /* FIXME: BROKEN! return convertCharSet("UTF-8", "HTML", string); } */
+static void common_buffer_parse_error(void *ctxt, const gchar * msg, ...);
 
 /* Conversion function which should be applied to all read XML strings, 
    to ensure proper UTF8. This is because we use libxml2 in recovery
@@ -130,7 +77,7 @@ gchar * convertToHTML(gchar * string) { return string; } /* FIXME: BROKEN! retur
    
    The valid or a corrected string is returned. The original XML 
    string is modified (FIXME: not sure if this is good). */
-gchar * utf8_fix(xmlChar *string) {
+gchar * common_utf8_fix(xmlChar *string) {
 	const gchar	*invalid_offset;
 
 	if(NULL == string)
@@ -155,24 +102,194 @@ gchar * utf8_fix(xmlChar *string) {
 	return string;
 }
 
-gchar * extractHTMLNode(xmlNodePtr cur, gboolean children) {
+long common_parse_long(gchar *str, long def) {
+	long num;
+
+	if(str == NULL)
+		return def;
+	if(0 == (sscanf(str,"%ld",&num)))
+		num = def;
+	
+	return num;
+}
+
+static xmlDocPtr common_parse_html(const gchar *html, gint len) {
+	xmlDocPtr out = NULL;
+	
+	g_assert(html != NULL);
+	g_assert(len >= 0);
+	
+
+	/* Note: NONET is not implemented so it will return an error
+	   because it doesn't know how to handle NONET. But, it might
+	   learn in the future. */
+	out = htmlReadMemory(html, len, NULL, "utf-8", HTML_PARSE_RECOVER | HTML_PARSE_NONET |
+	                     ((debug_level & DEBUG_HTML)?0:(HTML_PARSE_NOERROR | HTML_PARSE_NOWARNING)));
+	return out;
+}
+
+static xmlNodePtr common_html_doc_find_body(xmlDocPtr doc) {
+	xmlXPathContextPtr xpathCtxt = NULL;
+	xmlXPathObjectPtr xpathObj = NULL;
+	xmlNodePtr node = NULL;
+	xpathCtxt = xmlXPathNewContext(doc);
+	if(!xpathCtxt) goto error;
+
+	xpathObj = xmlXPathEvalExpression("/html/body", xpathCtxt);
+	if(!xpathObj) goto error;
+	if(!xpathObj->nodesetval->nodeMax) goto error;
+	
+	node = xpathObj->nodesetval->nodeTab[0];
+ error:
+	if (xpathObj) xmlXPathFreeObject(xpathObj);
+	if (xpathCtxt) xmlXPathFreeContext(xpathCtxt);
+	return node;
+}
+
+/* Extract XHTML from the children of the passed node. */
+gchar * extractHTMLNode(xmlNodePtr cur, gint xhtmlMode, const gchar *defaultBase) {
 	xmlBufferPtr	buf;
+	xmlChar         *xml_base = NULL;
 	gchar		*result = NULL;
 
-	buf = xmlBufferCreate();
-	if(children) {
-		cur = cur->xmlChildrenNode;
-		while (cur != NULL) {
-			xmlNodeDump(buf, cur->doc, cur, 0, 0);
-			cur = cur->next;
-		}
-	} else {
-		xmlNodeDump(buf, cur->doc, cur, 0, 0);
+	/* Create the new document and add the div tag*/
+	xmlDocPtr newDoc = xmlNewDoc( BAD_CAST "1.0" );;
+	xmlNodePtr divNode = xmlNewNode(NULL, BAD_CAST "div");;
+	xmlDocSetRootElement( newDoc, divNode );
+	xmlNewNs(divNode, BAD_CAST "http://www.w3.org/1999/xhtml", NULL);
+
+	/* Set the xml:base  of the div tag */
+	xml_base = xmlNodeGetBase(cur->doc, cur);
+	if(xml_base) {
+		xmlNodeSetBase( divNode, xml_base );
+		xmlFree(xml_base);
 	}
+	else if(defaultBase)
+		xmlNodeSetBase( divNode, defaultBase);
+	
+	if(xhtmlMode == 0) { /* Read escaped HTML and convert to XHTML, placing in a div tag */
+		xmlDocPtr oldDoc;
+		xmlNodePtr copiedNodes = NULL;
+		xmlChar *escapedhtml;
+		
+		/* Parse the HTML into oldDoc*/
+		escapedhtml = xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1);//xmlNodeDump(tmpBuf, cur->doc, cur, 0, 0);
+		if(escapedhtml) {
+			escapedhtml = g_strstrip(escapedhtml);	/* stripping whitespaces to make empty string detection easier */
+			if(*escapedhtml) {			/* never process empty content, xmlDocCopy() doesn't like it... */
+				xmlNodePtr body;
+				oldDoc = common_parse_html(escapedhtml, strlen(escapedhtml));
+				body = common_html_doc_find_body(oldDoc);
+
+				if(body) {
+					/* Copy in the html tags */
+					copiedNodes = xmlDocCopyNodeList(newDoc, body->xmlChildrenNode);
+					// FIXME: is the above correct? Why only operate on the first child node?
+					// It might be unproblematic because all content is wrapped in a <div>...
+					xmlAddChildList(divNode, copiedNodes);
+				}
+				xmlFreeDoc(oldDoc);
+				xmlFree(escapedhtml);
+			}
+		}
+	} else if(xhtmlMode == 1 || xhtmlMode == 2){ /* Read multiple XHTML tags and embed in div tag */
+		xmlNodePtr copiedNodes = xmlDocCopyNodeList( newDoc, cur->xmlChildrenNode);
+		xmlAddChildList(divNode, copiedNodes);
+	}
+	
+	buf = xmlBufferCreate();
+	xmlNodeDump(buf, newDoc, xmlDocGetRootElement(newDoc), 0, 0 );
+	
 	if(xmlBufferLength(buf) > 0)
 		result = xmlCharStrdup(xmlBufferContent(buf));
 
 	xmlBufferFree(buf);
+	xmlFreeDoc(newDoc);
+	return result;
+}
+
+gchar * common_strip_dhtml(const gchar *html) {
+	gchar *tmp;
+
+	// FIXME: move to XSLT stylesheet that post processes
+	// generated XHTML. The solution below might break harmless
+	// escaped HTML.
+	
+	/* remove some nasty DHTML stuff from the given HTML content */
+	tmp = g_strdup(html);
+	tmp = common_strreplace(tmp, " onload=", " no_onload=");
+	tmp = common_strreplace(tmp, " onLoad=", " no_onLoad=");
+	tmp = common_strreplace(tmp, "script>", "no_script>");		
+	tmp = common_strreplace(tmp, "<script ", "<no_script ");
+	tmp = common_strreplace(tmp, "<meta ", "<no_meta ");
+	tmp = common_strreplace(tmp, "/meta>", "/no_meta>");
+	tmp = common_strreplace(tmp, "<iframe ", "<no_iframe ");
+	tmp = common_strreplace(tmp, "/iframe>", "/no_iframe>");
+	
+	return tmp;
+}
+
+/* Convert the given string to proper XHTML content.*/
+gchar * common_text_to_xhtml(const gchar *sourceText) {
+	gchar		*text, *result = NULL;
+	xmlDocPtr	doc = NULL;
+	xmlBufferPtr	buf;
+	xmlNodePtr 	body, cur;
+	
+	if(!sourceText)
+		return g_strdup("");
+	
+	text = g_strstrip(g_strdup(sourceText));	/* stripping whitespaces to make empty string detection easier */
+	if(*text) {	
+		doc = common_parse_html(text, strlen(text));
+		
+		buf = xmlBufferCreate();
+		body = common_html_doc_find_body(doc);
+		if(body) {
+			cur = body->xmlChildrenNode;
+			while(cur) {
+				xmlNodeDump(buf, doc, cur, 0, 0 );
+				cur = cur->next;
+			}
+		}
+
+		if(xmlBufferLength(buf) > 0)
+			result = xmlCharStrdup(xmlBufferContent(buf));
+		else
+			result = g_strdup("");
+
+		xmlBufferFree(buf);
+		xmlFreeDoc(doc);
+		g_free(text);
+	} else {
+		result = text;
+	}
+	
+	return result;
+}
+
+gboolean common_is_well_formed_xhtml(const gchar *data) {
+	gchar		*xml;
+	gboolean	result;
+	errorCtxtPtr	errors;
+	xmlDocPtr	doc;
+
+	if(!data)
+		return FALSE;
+	
+	errors = g_new0(struct errorCtxt, 1);
+	errors->msg = g_string_new(NULL);
+
+	xml = g_strdup_printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n<test>%s</test>", data);
+	
+	doc = common_parse_xml(xml, strlen(xml), FALSE, errors);
+	if(doc)
+		xmlFreeDoc(doc);
+		
+	g_free(xml);
+	g_string_free(errors->msg, TRUE);
+	result = (0 == errors->errorCount);
+	g_free(errors);
 	
 	return result;
 }
@@ -225,7 +342,7 @@ static gchar * unmarkupize(gchar *string, void(*parse)(gchar *string, result_buf
 	if(NULL == string)
 		return NULL;
 		
-	string = utf8_fix(string);
+	string = common_utf8_fix(string);
 
 	/* only do something if there are any entities or tags */
 	if(NULL == (strpbrk(string, "&<>")))
@@ -252,12 +369,6 @@ gchar * unxmlize(gchar * string) { return unmarkupize(string, _unxmlize); }
 
 #define MAX_PARSE_ERROR_LINES	10
 
-/** used by bufferParseError to keep track of error messages */
-typedef struct errorCtxt {
-	gchar	*buffer;
-	gint	errorCount;
-} *errorCtxtPtr;
-
 /**
  * Error buffering function to be registered by 
  * xmlSetGenericErrorFunc(). This function is called on
@@ -267,13 +378,11 @@ typedef struct errorCtxt {
  * @param	ctxt	error context
  * @param	msg	printf like format string 
  */
-static void bufferParseError(void *ctxt, const gchar * msg, ...) {
+static void common_buffer_parse_error(void *ctxt, const gchar * msg, ...) {
 	va_list		params;
 	errorCtxtPtr	errors = (errorCtxtPtr)ctxt;
 	gchar		*newmsg;
 	gchar		*tmp;
-
-	g_assert(NULL != errors);
 	
 	if(MAX_PARSE_ERROR_LINES > errors->errorCount++) {
 		
@@ -282,23 +391,17 @@ static void bufferParseError(void *ctxt, const gchar * msg, ...) {
 		va_end(params);
 
 		g_assert(NULL != newmsg);
-		newmsg = utf8_fix(newmsg);
+		newmsg = common_utf8_fix(newmsg);
 		tmp = g_markup_escape_text(newmsg, -1);
 		g_free(newmsg);
 		newmsg = tmp;
 	
-		addToHTMLBufferFast(&errors->buffer, "<pre>");
-		addToHTMLBufferFast(&errors->buffer, newmsg);
-		addToHTMLBufferFast(&errors->buffer, "</pre>");
-
+		g_string_append_printf(errors->msg, "<pre>%s</pre", newmsg);
 		g_free(newmsg);
 	}
 	
-	if(MAX_PARSE_ERROR_LINES == errors->errorCount) {
-		newmsg = g_strdup_printf("%s<br/>%s", errors->buffer, _("[Parser error output was truncated!]"));
-		g_free(errors->buffer);
-		errors->buffer = newmsg;
-	}
+	if(MAX_PARSE_ERROR_LINES == errors->errorCount)
+		g_string_append_printf(errors->msg, "<br />%s", _("[There were more errors. Output was truncated!]"));
 }
 
 static xmlDocPtr entities = NULL;
@@ -324,7 +427,7 @@ xmlEntityPtr common_process_entities(void *ctxt, const xmlChar *name) {
 			entity = (xmlEntityPtr)g_new0(xmlEntity, 1);
 			entity->type = XML_ENTITY_DECL;
 			entity->name = name;
-			entity->orig = tmp;	/* ??? */
+			entity->orig = NULL;
 			entity->content = tmp;
 			entity->length = g_utf8_strlen(tmp, -1);
 			entity->etype = XML_INTERNAL_PREDEFINED_ENTITY;
@@ -336,62 +439,263 @@ xmlEntityPtr common_process_entities(void *ctxt, const xmlChar *name) {
 	return entity;
 }
 
-/**
- * Common function to create a XML DOM object from a given
- * XML buffer. This function sets up a parser context,
- * enables recovery mode and sets up the error handler.
- * 
- * The function returns a XML document pointer or NULL
- * if the document could not be read. It also sets 
- * errormsg to the last error messages on parsing
- * errors. 
- *
- * @param data		XML source
- * @param dataLength the length of the data string in bytes
- * @param errormsg	error buffer
- *
- * @return XML document
- */
-xmlDocPtr parseBuffer(gchar *data, size_t dataLength, gchar **errormsg) {
+xmlNodePtr common_xpath_find(xmlNodePtr node, gchar *expr) {
+	xmlNodePtr	result = NULL;
+	
+	if(node && node->doc) {
+		xmlXPathContextPtr xpathCtxt = NULL;
+		xmlXPathObjectPtr xpathObj = NULL;
+		
+		if(NULL != (xpathCtxt = xmlXPathNewContext(node->doc))) {
+			xpathCtxt->node = node;
+			xpathObj = xmlXPathEval(expr, xpathCtxt);
+		}
+		
+		if(xpathObj && !xmlXPathNodeSetIsEmpty(xpathObj->nodesetval))
+			result = xpathObj->nodesetval->nodeTab[0];
+		
+		if(xpathObj) xmlXPathFreeObject(xpathObj);
+		if(xpathCtxt) xmlXPathFreeContext(xpathCtxt);
+	}
+	return result;
+}
+
+gboolean common_xpath_foreach_match(xmlNodePtr node, gchar *expr, xpathMatchFunc func, gpointer user_data) {
+	
+	if(node && node->doc) {
+		xmlXPathContextPtr xpathCtxt = NULL;
+		xmlXPathObjectPtr xpathObj = NULL;
+		
+		if(NULL != (xpathCtxt = xmlXPathNewContext(node->doc))) {
+			xpathCtxt->node = node;
+			xpathObj = xmlXPathEval(expr, xpathCtxt);
+		}
+			
+		if(xpathObj && xpathObj->nodesetval && xpathObj->nodesetval->nodeMax) {
+			int	i;
+			for(i = 0; i < xpathObj->nodesetval->nodeNr; i++)
+				(*func)(xpathObj->nodesetval->nodeTab[i], user_data);
+		}
+		
+		if(xpathObj) xmlXPathFreeObject(xpathObj);
+		if(xpathCtxt) xmlXPathFreeContext(xpathCtxt);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+xmlDocPtr common_parse_xml(gchar *data, size_t length, gboolean recovery, errorCtxtPtr errCtx) {
 	xmlParserCtxtPtr	ctxt;
-	errorCtxtPtr		errors;
-	xmlDocPtr               doc;
+	xmlDocPtr		doc;
 	
 	g_assert(NULL != data);
-	
-	/* we don't like no data */
-	if(0 == dataLength) {
-		g_warning("parseBuffer(): Empty input!\n");
-		*errormsg = g_strdup("parseBuffer(): Empty input!\n");
-		return NULL;
-	}
-	
-	errors = g_new0(struct errorCtxt, 1);
-	xmlSetGenericErrorFunc(errors, (xmlGenericErrorFunc)bufferParseError);
+
 	ctxt = xmlNewParserCtxt();
 	ctxt->sax->getEntity = common_process_entities;
-	doc = xmlSAXParseMemory(ctxt->sax, data, dataLength, /* recovery = */ TRUE);
-	if (doc == NULL) {
-		g_warning("xmlReadMemory: Could not parse document!\n");
-		*errormsg = g_strdup_printf(_("xmlReadMemory(): Could not parse document:\n%s%s"), 
-		                            errors->buffer != NULL ? errors->buffer : "",
-		                            errors->buffer != NULL ? "\n" : "");
-		g_free(errors->buffer);
-		errors->buffer = *errormsg;
-	}
+	
+	if(errCtx)
+		xmlSetGenericErrorFunc(errCtx, (xmlGenericErrorFunc)common_buffer_parse_error);
+	
+	doc = xmlSAXParseMemory(ctxt->sax, data, length, /* recovery = */ recovery);
 	
 	/* This seems to reset the errorfunc to its default, so that the
 	   GtkHTML2 module is not unhappy because it also tries to call the
 	   errorfunc on occasion. */
 	xmlSetGenericErrorFunc(NULL, NULL);
-
-	*errormsg = errors->buffer;
-	g_free(errors);
+	
 	xmlFreeParserCtxt(ctxt);
 	
 	return doc;
 }
 
+xmlDocPtr common_parse_xml_feed(feedParserCtxtPtr fpc) {
+	errorCtxtPtr		errors;
+		
+	g_assert(NULL != fpc->data);
+	g_assert(NULL != fpc->feed);
+	g_assert(NULL != fpc->itemSet);
+	
+	fpc->itemSet->valid = FALSE;
+	
+	/* we don't like no data */
+	if(0 == fpc->dataLength) {
+		debug1(DEBUG_PARSING, "common_parse_xml_feed(): empty input while parsing \"%s\"!", fpc->node->title);
+		g_string_append(fpc->feed->parseErrors, "Empty input!\n");
+		return NULL;
+	}
+
+	errors = g_new0(struct errorCtxt, 1);
+	errors->msg = fpc->feed->parseErrors;
+	
+	fpc->doc = common_parse_xml(fpc->data, (size_t)fpc->dataLength, fpc->recovery, errors);
+	if(!fpc->doc) {
+		debug1(DEBUG_PARSING, "xmlReadMemory: could not parse feed \"%s\"!\n", fpc->node->title);
+		g_string_prepend(fpc->feed->parseErrors, _("XML Parser: Could not parse document:\n"));
+		g_string_append(fpc->feed->parseErrors, "\n");
+	}
+
+	fpc->itemSet->valid = (errors->errorCount > 0);
+	g_free(errors);
+	
+	return fpc->doc;
+}
+
+/* The following slightly modified function was originally taken 
+ * from GConf 2.16.0 backends/xml-dir.c: Copyright (C) 1999, 2000 Red Hat Inc. */
+
+/* for info on why this is used rather than xmlDocDump or xmlSaveFile
+ * and friends, see http://bugzilla.gnome.org/show_bug.cgi?id=108329 */
+gint common_save_xml(xmlDocPtr doc, gchar *filename) {
+	FILE	*fp;
+	char	*xmlbuf;
+	int	fd, n;
+
+	fp = g_fopen(filename, "w");
+	if(NULL == fp)
+		return -1;
+  
+	xmlDocDumpFormatMemory(doc, (xmlChar **)&xmlbuf, &n, TRUE);
+	if(n <= 0) {
+		errno = ENOMEM;
+		return -1;
+	}
+
+	if(fwrite(xmlbuf, sizeof (xmlChar), n, fp) < n)	{
+		xmlFree (xmlbuf);
+		return -1;
+	}
+
+	xmlFree (xmlbuf);
+
+	/* From the fflush(3) man page:
+	*
+	* Note that fflush() only flushes the user space buffers provided by the
+	* C library. To ensure that the data is physically stored on disk the
+	* kernel buffers must be flushed too, e.g. with sync(2) or fsync(2).
+	*/
+
+	/* flush user-space buffers */
+	if (fflush (fp) != 0)
+		return -1;
+
+	if ((fd = fileno (fp)) == -1)
+		return -1;
+
+#ifdef HAVE_FSYNC
+	/* sync kernel-space buffers to disk */
+	if (fsync (fd) == -1)
+		return -1;
+#endif
+
+	fclose(fp);
+
+	return 0;
+}
+
+#define	TIMESTRLEN	256
+
+gchar * common_format_date(time_t date, const gchar *date_format) {
+	gchar		*result, *formbuf;
+	struct tm	date_tm;
+	
+	if (date == 0) {
+		return g_strdup ("");
+	}
+	
+	localtime_r (&date, &date_tm);
+	
+	result = g_new0(gchar, TIMESTRLEN);
+	formbuf = g_locale_from_utf8 (date_format, -1, NULL, NULL, NULL);
+	strftime(result, TIMESTRLEN, formbuf, &date_tm);
+	g_free(formbuf);
+	return result;
+}
+
+/* This function is originally from the Evolution 2.6.2 code (e-cell-date.c) */
+gchar * common_format_nice_date(time_t date) {
+	time_t nowdate = time(NULL);
+	time_t yesdate;
+	struct tm then, now, yesterday;
+	gchar *temp, *buf, *formbuf;
+	gboolean done = FALSE;
+	
+	if (date == 0) {
+		return g_strdup ("");
+	}
+	
+	buf = g_new0(gchar, TIMESTRLEN + 1);
+
+	localtime_r (&date, &then);
+	localtime_r (&nowdate, &now);
+
+/*	if (nowdate - date < 60 * 60 * 8 && nowdate > date) {
+		strftime (buf, TIMESTRLEN, _("%l:%M %p"), &then);
+		done = TRUE;
+	}*/
+
+	if (!done) {
+		if (then.tm_mday == now.tm_mday &&
+		    then.tm_mon == now.tm_mon &&
+		    then.tm_year == now.tm_year) {
+		    	/* translation hint: date format for today, reorder format codes as necessary */
+			formbuf = g_locale_from_utf8 (_("Today %l:%M %p"), -1, NULL, NULL, NULL);
+			strftime (buf, TIMESTRLEN, formbuf, &then);
+			g_free(formbuf);
+			done = TRUE;
+		}
+	}
+	if (!done) {
+		yesdate = nowdate - 60 * 60 * 24;
+		localtime_r (&yesdate, &yesterday);
+		if (then.tm_mday == yesterday.tm_mday &&
+		    then.tm_mon == yesterday.tm_mon &&
+		    then.tm_year == yesterday.tm_year) {
+		    	/* translation hint: date format for yesterday, reorder format codes as necessary */
+			formbuf = g_locale_from_utf8 (_("Yesterday %l:%M %p"), -1, NULL, NULL, NULL);
+			strftime (buf, TIMESTRLEN, formbuf, &then);
+			g_free(formbuf);
+			done = TRUE;
+		}
+	}
+	if (!done) {
+		int i;
+		for (i = 2; i < 7; i++) {
+			yesdate = nowdate - 60 * 60 * 24 * i;
+			localtime_r (&yesdate, &yesterday);
+			if (then.tm_mday == yesterday.tm_mday &&
+			    then.tm_mon == yesterday.tm_mon &&
+			    then.tm_year == yesterday.tm_year) {
+			    	/* translation hint: date format for dates older than 2 days but not older than a week, reorder format codes as necessary */
+				formbuf = g_locale_from_utf8 (_("%a %l:%M %p"), -1, NULL, NULL, NULL);
+				strftime (buf, TIMESTRLEN, formbuf, &then);
+				g_free(formbuf);
+				done = TRUE;
+				break;
+			}
+		}
+	}
+	if (!done) {
+		if (then.tm_year == now.tm_year) {
+			/* translation hint: date format for dates older than a week but from this year, reorder format codes as necessary */
+			formbuf = g_locale_from_utf8 (_("%b %d %l:%M %p"), -1, NULL, NULL, NULL);
+			strftime (buf, TIMESTRLEN, formbuf, &then);
+			g_free(formbuf);
+		} else {
+			/* translation hint: date format for dates from the last years, reorder format codes as necessary */
+			formbuf = g_locale_from_utf8 (_("%b %d %Y"), -1, NULL, NULL, NULL);
+			strftime (buf, TIMESTRLEN, formbuf, &then);
+			g_free(formbuf);
+		}
+	}
+
+	temp = buf;
+	while ((temp = strstr (temp, "  "))) {
+		memmove (temp, temp + 1, strlen (temp));
+	}
+	temp = g_strstrip (buf);
+	return temp;
+}
+
 /* converts a ISO 8601 time string to a time_t value */
 time_t parseISO8601Date(gchar *date) {
 	struct tm	tm;
@@ -549,15 +853,15 @@ time_t parseRFC822Date(gchar *date) {
 	else if(NULL != (pos = strptime((const char *)date, " %d %b %y %H:%M", &tm)))
 		success = TRUE;
 	
-	while(pos != NULL && *pos != '\0' && isspace((int)*pos))       /* skip whitespaces before timezone */
+	while(pos && *pos != '\0' && isspace((int)*pos))       /* skip whitespaces before timezone */
 		pos++;
 	
-	if(NULL != oldlocale) {
+	if(oldlocale) {
 		setlocale(LC_TIME, oldlocale);	/* and reset it again */
 		g_free(oldlocale);
 	}
 	
-	if(TRUE == success) {
+	if(success) {
 		if((time_t)(-1) != (t = mktime(&tm))) {
 			/* GMT time, with no daylight savings time
 			   correction. (Usually, there is no daylight savings
@@ -574,63 +878,52 @@ time_t parseRFC822Date(gchar *date) {
 	return 0;
 }
 
-void initCachePath(void) {
-	gchar *cachePath;
-	gchar *feedCachePath;
-	gchar *faviconCachePath;
+static void common_check_dir(gchar *path) {
 
-	lifereaUserPath = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".liferea", g_get_home_dir());
-	if(!g_file_test(lifereaUserPath, G_FILE_TEST_IS_DIR)) {
-		if(0 != mkdir(lifereaUserPath, S_IRUSR | S_IWUSR | S_IXUSR)) {
-			g_error(_("Cannot create cache directory %s!"), lifereaUserPath);
+	if(!g_file_test(path, G_FILE_TEST_IS_DIR)) {
+		if(0 != g_mkdir_with_parents(path, S_IRUSR | S_IWUSR | S_IXUSR)) {
+			g_error(_("Cannot create cache directory \"%s\"!"), path);
 		}
 	}
-	
-	cachePath = g_strdup_printf("%s" G_DIR_SEPARATOR_S "cache", lifereaUserPath);
-	if(!g_file_test(cachePath, G_FILE_TEST_IS_DIR)) {
-		if(0 != mkdir(cachePath, S_IRUSR | S_IWUSR | S_IXUSR)) {
-			g_error(_("Cannot create cache directory %s!"), cachePath);
-		}
-	}
-	feedCachePath = g_strdup_printf("%s" G_DIR_SEPARATOR_S "feeds", cachePath);
-	if(!g_file_test(feedCachePath, G_FILE_TEST_IS_DIR)) {
-		if(0 != mkdir(feedCachePath, S_IRUSR | S_IWUSR | S_IXUSR)) {
-			g_error(_("Cannot create cache directory %s!"), feedCachePath);
-		}
-	}
-
-	faviconCachePath = g_strdup_printf("%s" G_DIR_SEPARATOR_S "favicons", cachePath);
-	if(!g_file_test(faviconCachePath, G_FILE_TEST_IS_DIR)) {
-		if(0 != mkdir(faviconCachePath, S_IRUSR | S_IWUSR | S_IXUSR)) {
-			g_error(_("Cannot create cache directory %s!"), faviconCachePath);
-		}
-	}
-
-	g_free(cachePath);
-	g_free(feedCachePath);
-	g_free(faviconCachePath);
-
+	g_free(path);
 }
 
-gchar * common_get_cache_path(void) {
+static void common_init_cache_path(void) {
+	gchar *cachePath;
+
+	lifereaUserPath = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".liferea_1.2", g_get_home_dir());
+	cachePath = g_strdup_printf("%s" G_DIR_SEPARATOR_S "cache", lifereaUserPath);
+
+	common_check_dir(g_strdup(lifereaUserPath));
+	common_check_dir(g_strdup(cachePath));
+	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "feeds", cachePath));
+	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "favicons", cachePath));
+	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "plugins", cachePath));
+	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "scripts", cachePath));
+
+	g_free(cachePath);
+	/* lifereaUserPath reused globally */
+}
+
+const gchar * common_get_cache_path(void) {
 	
-	if(NULL == lifereaUserPath)
-		initCachePath();
+	if(!lifereaUserPath)
+		common_init_cache_path();
 		
 	return lifereaUserPath;
 }
 
-gchar * common_create_cache_filename( const gchar *folder, const gchar *key, const gchar *extension) {
-	gchar *filename;
+gchar * common_create_cache_filename(const gchar *folder, const gchar *filename, const gchar *extension) {
+	gchar *result;
 
-	filename = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s%s%s%s%s", common_get_cache_path(),
-	                           (folder != NULL) ? folder : "",
-	                           (folder != NULL) ? G_DIR_SEPARATOR_S : "",
-	                           key,
-	                           (extension != NULL)? "." : "",
-	                           (extension != NULL)? extension : "");
+	result = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s%s%s%s%s", common_get_cache_path(),
+	                         (folder != NULL) ? folder : "",
+	                         (folder != NULL) ? G_DIR_SEPARATOR_S : "",
+	                         filename,
+	                         (extension != NULL)? "." : "",
+	                         (extension != NULL)? extension : "");
 
-	return filename;
+	return result;
 }
 
 static gchar * byte_to_hex(unsigned char nr) {
@@ -640,7 +933,7 @@ static gchar * byte_to_hex(unsigned char nr) {
 	return result;
 }
 
-gchar * encode_uri_string(gchar *string) {
+gchar * common_encode_uri_string(gchar *string) {
 	gchar		*newURIString;
 	gchar		*hex, *tmp = NULL;
 	int		i, j, len, bytes;
@@ -709,8 +1002,12 @@ xmlChar * common_uri_escape(const xmlChar *url) {
 	return result;	
 }
 
-/* to correctly escape and expand URLs, does not touch the
-   passed strings */
+xmlChar * common_uri_unescape(const xmlChar *url) {
+
+	return xmlURIUnescapeString(url, -1, NULL);
+}
+
+/* to correctly escape and expand URLs */
 xmlChar * common_build_url(const gchar *url, const gchar *baseURL) {
 	xmlChar	*escapedURL, *absURL, *escapedBaseURL;
 
@@ -757,9 +1054,8 @@ const gchar * common_get_direction_mark(gchar *text) {
 	}
 }
 
-#ifndef HAVE_STRSEP
 /* code taken from glibc-2.2.1/sysdeps/generic/strsep.c */
-char* strsep (char **stringp, const char *delim) {
+char * common_strsep (char **stringp, const char *delim) {
 	char *begin, *end;
 
 	begin = *stringp;
@@ -800,12 +1096,10 @@ char* strsep (char **stringp, const char *delim) {
 		*stringp = NULL;
 	return begin;
 }
-#endif /*HAVE_STRSEP*/
 
 /* Taken from gaim 24 June 2004, copyrighted by the gaim developers
-   under the GPL, etc.... */
-gchar *strreplace(const char *string, const char *delimiter,
-			   const char *replacement) {
+   under the GPL, etc.... It was slightly modified to free the passed string */
+gchar * common_strreplace(gchar *string, const gchar *delimiter, const gchar *replacement) {
 	gchar **split;
 	gchar *ret;
 
@@ -816,6 +1110,7 @@ gchar *strreplace(const char *string, const char *delimiter,
 	split = g_strsplit(string, delimiter, 0);
 	ret = g_strjoinv(replacement, split);
 	g_strfreev(split);
+	g_free(string);
 
 	return ret;
 }
@@ -825,7 +1120,7 @@ typedef unsigned chartype;
 /* strcasestr is Copyright (C) 1994, 1996-2000, 2004 Free Software
    Foundation, Inc.  It was taken from the GNU C Library, which is
    licenced under the GPL v2.1 or (at your option) newer version. */
-char *liferea_strcasestr (const char *phaystack, const char *pneedle)
+char *common_strcasestr (const char *phaystack, const char *pneedle)
 {
 	register const unsigned char *haystack, *needle;
 	register chartype b, c;
@@ -835,7 +1130,7 @@ char *liferea_strcasestr (const char *phaystack, const char *pneedle)
 
 	b = tolower(*needle);
 	if (b != '\0') {
-		haystack--;                               /* possible ANSI violation */
+		haystack--;             /* possible ANSI violation */
 		do {
 			c = *++haystack;
 			if (c == '\0')
