@@ -31,14 +31,12 @@
 #include "support.h"
 #include "callbacks.h"
 #include "update.h"
-#include "feed.h"
-#include "folder.h"
 #include "common.h"
 #include "conf.h"
 #include "debug.h"
-#include "ui_tray.h"
-#include "ui_htmlview.h"
-#include "ui_mainwindow.h"
+#include "ui/ui_tray.h"
+#include "ui/ui_htmlview.h"
+#include "ui/ui_mainwindow.h"
 
 #define MAX_GCONF_PATHLEN	256
 
@@ -48,11 +46,8 @@
 
 static GConfClient	*client;
 
-static guint feedlist_save_timer;
-static gboolean feedlistLoading = FALSE;	/* flag set during initial feed list loading */
-
 /* configuration values for the SnowNews HTTP code used from within netio.c */
-int	NET_TIMEOUT;
+int	NET_TIMEOUT = 30;
 char 	*useragent = NULL;
 char	*proxyname = NULL;
 char	*proxyusername = NULL;
@@ -110,7 +105,7 @@ void conf_init() {
 	conf_proxy_reset_settings_cb(NULL, 0, NULL, NULL);
 
 	if(0 == (NET_TIMEOUT = getNumericConfValue(NETWORK_TIMEOUT)))
-		NET_TIMEOUT = 60;	/* default network timeout 30s */
+		NET_TIMEOUT = 30;	/* default network timeout 30s */
 }
 
 /* maybe called several times to reload configuration */
@@ -143,7 +138,7 @@ static void conf_toolbar_style_settings_cb(GConfClient *client, guint cnxn_id, G
 		
 		value = gconf_entry_get_value(entry);
 		if (value != NULL && value->type == GCONF_VALUE_STRING)
-			ui_mainwindow_set_toolbar_style(GTK_WINDOW(mainwindow), gconf_value_get_string(value));
+			ui_mainwindow_set_toolbar_style(gconf_value_get_string(value));
 	}
 }
 
@@ -205,82 +200,6 @@ static void conf_proxy_reset_settings_cb(GConfClient *client, guint cnxn_id, GCo
 		  proxypassword != NULL ? proxypassword : "NULL");
 }
 
-/*----------------------------------------------------------------------*/
-/* feed entry handling							*/
-/*----------------------------------------------------------------------*/
-
-gchar* conf_new_id() {
-	int		i;
-	gchar		*id, *filename;
-	gboolean	already_used;
-	
-	id = g_new0(gchar, 10);
-	do {
-		for(i=0;i<7;i++)
-			id[i] = (char)g_random_int_range('a', 'z');
-		id[7] = '\0';
-		
-		filename = common_create_cache_filename("cache" G_DIR_SEPARATOR_S "feeds", id, NULL);
-		already_used = g_file_test(filename, G_FILE_TEST_EXISTS);
-		g_free(filename);
-	} while(already_used);
-	
-	return id;
-}
-
-/*----------------------------------------------------------------------*/
-/* config loading on startup						*/
-/*----------------------------------------------------------------------*/
-
-void conf_load_subscriptions(void) {
-	gchar	*filename;
-	
-	feedlistLoading = TRUE;
-	filename = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".liferea" G_DIR_SEPARATOR_S "feedlist.opml", g_get_home_dir());
-	if(!g_file_test(filename, G_FILE_TEST_EXISTS)) {
-		/* if there is no feedlist.opml we provide a default feed list */
-		g_free(filename);
-		/* "feedlist.opml" is translatable so that translators can provide a localized default feed list */
-		filename = g_strdup_printf(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "opml" G_DIR_SEPARATOR_S "%s", _("feedlist.opml"));
-	}
-	import_OPML_feedlist(filename, NULL, FALSE, TRUE);
-	g_free(filename);
-	feedlistLoading = FALSE;
-}
-
-void conf_feedlist_save() {
-	gchar *filename, *filename_real;
-	
-	if(feedlistLoading)
-		return;
-
-	debug_enter("conf_feedlist_save");
-	filename = g_strdup_printf("%s" G_DIR_SEPARATOR_S "feedlist.opml~", common_get_cache_path());
-
-	if(0 == export_OPML_feedlist(filename, TRUE)) {
-		filename_real = g_strdup_printf("%s" G_DIR_SEPARATOR_S "feedlist.opml", common_get_cache_path());
-		if(rename(filename, filename_real) < 0)
-			g_warning(_("Error renaming %s to %s\n"), filename, filename_real);
-		g_free(filename_real);
-	}
-	g_free(filename);
-	debug_exit("conf_feedlist_save");
-}
-
-static gboolean conf_feedlist_schedule_save_cb(gpointer user_data) {
-
-	conf_feedlist_save();
-	feedlist_save_timer = 0;
-	return FALSE;
-}
-
-void conf_feedlist_schedule_save() {
-
-	if(!feedlistLoading && !feedlist_save_timer) {
-		debug0(DEBUG_CONF, "Scheduling feedlist save");
-		feedlist_save_timer = g_timeout_add(5000, conf_feedlist_schedule_save_cb, NULL);
-	}
-}
 
 /*----------------------------------------------------------------------*/
 /* generic configuration access methods					*/
@@ -305,7 +224,7 @@ gboolean getBooleanConfValue(gchar *valuename) {
 
 	g_assert(valuename != NULL);
 
-	value = gconf_client_get_without_default(client, valuename, NULL);
+	value = gconf_client_get(client, valuename, NULL);
 	if(NULL == value) {
 		setBooleanConfValue(valuename, FALSE);
 		result = FALSE;
@@ -336,7 +255,7 @@ gchar * getStringConfValue(gchar *valuename) {
 
 	g_assert(valuename != NULL);
 		
-	value = gconf_client_get_without_default(client, valuename, NULL);
+	value = gconf_client_get(client, valuename, NULL);
 	if(NULL == value) {
 		result = g_strdup("");
 	} else {
@@ -366,7 +285,7 @@ gint getNumericConfValue(gchar *valuename) {
 
 	g_assert(valuename != NULL);
 		
-	value = gconf_client_get_without_default(client, valuename, NULL);
+	value = gconf_client_get(client, valuename, NULL);
 	if(NULL != value) {
 		result = gconf_value_get_int(value);
 		gconf_value_free(value);
