@@ -1,7 +1,7 @@
 /**
  * @file conf.h Liferea configuration (gconf access)
  *
- * Copyright (C) 2003-2005 Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004,2005 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,6 @@
 #define _CONF_H
 
 #include <glib.h>
-#include "folder.h"
 
 /* browsing settings */
 #define BROWSE_INSIDE_APPLICATION	"/apps/liferea/browse-inside-application"
@@ -34,16 +33,23 @@
 #define BROWSER_MODULE			"/apps/liferea/browser-module"
 #define GNOME_DEFAULT_BROWSER_COMMAND	"gnome-open '%s'"
 #define DEFAULT_BROWSER_COMMAND		"mozilla '%s'"
-#define DEFAULT_FONT			"/desktop/gnome/interface/font_name"
+#define DEFAULT_FONT			"/desktop/gnome/interface/document_font_name"
 #define USER_FONT			"/apps/liferea/browser-font"
 #define REFOCUS_TIMEOUT			"/apps/liferea/refocus-timeout"
 #define DISABLE_JAVASCRIPT		"/apps/liferea/disable-javascript"
+#define SOCIAL_BM_SITE			"/apps/liferea/social-bm-site"
+#define SOCIAL_BM_HIDE_LINK		"/apps/liferea/social-bm-hide-link"
+#define SEARCH_ENGINE			"/apps/liferea/search-engine"
+#define SEARCH_ENGINE_HIDE_LINK		"/apps/liferea/search-engine-hide-link"
 
 /* enclosure handling */
 #define ENCLOSURE_DOWNLOAD_TOOL		"/apps/liferea/enclosure-download-tool"
 #define ENCLOSURE_DOWNLOAD_PATH		"/apps/liferea/enclosure-download-path"
 
 /* item list settings */
+#define DATE_FORMAT			"/apps/liferea/date-format"
+
+// FIXME: remove me!
 #define TIME_FORMAT			"/apps/liferea/timeformat"
 #define TIME_FORMAT_MODE		"/apps/liferea/timeformatmode"
 
@@ -53,6 +59,8 @@
 #define STARTUP_FEED_ACTION		"/apps/liferea/startup_feed_action"
 #define UPDATE_THREAD_CONCURRENCY	"/apps/liferea/update-thread-concurrency"
 #define KEEP_FEEDS_IN_MEMORY		"/apps/liferea/keep-feeds-in-memory"
+#define DISABLE_SUBSCRIPTION_PIPE	"/apps/liferea/disable-subscription-pipe"
+#define ENABLE_FETCH_RETRIES		"/apps/liferea/enable-fetch-retries"
 
 /* folder handling settings */
 #define FOLDER_DISPLAY_MODE		"/apps/liferea/folder-display-mode"
@@ -60,6 +68,8 @@
 
 /* GUI settings and persistency values */
 #define SHOW_TRAY_ICON			"/apps/liferea/trayicon"
+#define SHOW_NEW_COUNT_IN_TRAY		"/apps/liferea/trayicon-new-count"
+#define DONT_MINIMIZE_TO_TRAY		"/apps/liferea/dont-minimize-to-tray"
 #define SHOW_POPUP_WINDOWS		"/apps/liferea/show-popup-windows"
 #define POPUP_PLACEMENT			"/apps/liferea/popup-placement"
 #define DISABLE_MENUBAR			"/apps/liferea/disable-menubar"
@@ -71,6 +81,7 @@
 #define LAST_WINDOW_MAXIMIZED		"/apps/liferea/last-window-maximized"
 #define LAST_VPANE_POS			"/apps/liferea/last-vpane-pos"
 #define LAST_HPANE_POS			"/apps/liferea/last-hpane-pos"
+#define LAST_WPANE_POS			"/apps/liferea/last-wpane-pos"
 #define LAST_ZOOMLEVEL			"/apps/liferea/last-zoomlevel"
 
 /* networking settings */
@@ -89,18 +100,9 @@
 void	conf_init(void);
 void	conf_load(void);
 
-/* feed/directory list entry manipulation methods */
-gchar*	conf_new_id();
-void	conf_load_subscriptions();
-void 	conf_feedlist_schedule_save();
-void 	conf_feedlist_save();
-
 /* methods to modify folder contents */
 GSList * getFeedKeyList(gchar *keyprefix);
 void 	setFeedKeyList(gchar *keyprefix, GSList *newlist);
-
-folderPtr feedlist_insert_help_folder(folderPtr parent, GTimeVal *lastPoll, GTimeVal *lastFaviconPoll);
-
 gchar * getFreeFeedKey(gchar *keyprefix);
 
 /* preferences configuration methods */
