@@ -1,8 +1,8 @@
 /**
  * @file gtkhtml2.c GtkHTML2 browser module implementation for Liferea
  *
- * Copyright (C) 2004 Nathan Conrad <conrad@bungled.net>
- * Copyright (C) 2003,2004 Lars Lindner <lars.lindner@gmx.net>  
+ * Copyright (C) 2004-2006 Nathan Conrad <conrad@bungled.net>
+ * Copyright (C) 2003-2006 Lars Lindner <lars.lindner@gmx.net>  
  * Copyright (C) 2004 Juho Snellman <jsnell@users.sourceforge.net>
  * 
  * Note large portions of this code (callbacks and html widget
@@ -38,15 +38,17 @@
 #include <string.h>
 #include <glib.h>
 #include <errno.h>
-#include "../common.h"
-#include "../ui_htmlview.h"
-#include "../support.h"
-#include "../callbacks.h"
-#include "../update.h"
-#include "../debug.h"
+#include "common.h"
+#include "ui/ui_htmlview.h"
+#include "support.h"
+#include "callbacks.h"
+#include "update.h"
+#include "debug.h"
 
 #define BUFFER_SIZE 8192
 
+#define EMPTY "<html><body></body></html>"
+
 /* points to the URL actually under the mouse pointer or is NULL */
 static gchar		*selectedURL = NULL;
 
@@ -54,21 +56,30 @@ static GdkCursor	*link_cursor = NULL;
 
 /* prototypes */
 static void link_clicked (HtmlDocument *doc, const gchar *url, gpointer data);
-static void launch_url(GtkWidget *widget, const gchar *url);
 static void gtkhtml2_scroll_to_top(GtkWidget *scrollpane);
 
 static int button_press_event(HtmlView *view, GdkEventButton *event, gpointer userdata) {
-
+	gboolean	safeURL = FALSE;
+	gboolean	isLocalDoc = FALSE;
+	
 	g_return_val_if_fail(view != NULL, FALSE);
 	g_return_val_if_fail(event != NULL, FALSE);
-
+	
 	if((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {
-		if(NULL == selectedURL) {
+	
+		/* source document in local filesystem? */
+		isLocalDoc = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(view->document), "localDocument"));
+
+		/* prevent launching local filesystem links */	
+		if(selectedURL)
+			safeURL = (NULL == strstr(selectedURL, "file://")) || isLocalDoc;
+
+		if(!selectedURL) {
 			gtk_menu_popup(GTK_MENU(make_html_menu()), NULL, NULL,
 				       NULL, NULL, event->button, event->time);
 		} else {
 			gdk_window_set_cursor(GDK_WINDOW(gtk_widget_get_parent_window(GTK_WIDGET(view))), NULL);
-			gtk_menu_popup(GTK_MENU(make_url_menu(selectedURL)), NULL, NULL,
+			gtk_menu_popup(GTK_MENU(make_url_menu(safeURL?selectedURL:"")), NULL, NULL,
 				       NULL, NULL, event->button, event->time);
 		}
 		g_free(selectedURL);
@@ -160,6 +171,7 @@ stream_cancel (HtmlStream *stream, gpointer user_data, gpointer cancel_data)
 	struct request *r = (struct request*)cancel_data;
 
 	debug1(DEBUG_UPDATE, "GtkHTML2: Canceling stream: %s", ((struct request*)user_data)->source);
+	update_cancel_requests((gpointer)stream);
 	
 	request_data_kill(r);
 }
@@ -174,14 +186,20 @@ static void gtkhtml2_url_request_received_cb(struct request *r) {
 }
 
 static void url_request(HtmlDocument *doc, const gchar *url, HtmlStream *stream, gpointer data) {
-	gchar		*absURL;
+	gchar		*absURL, *base;
+	gboolean	isLocalDoc;
 	
 	g_assert(NULL != stream);
+	
+	base = g_object_get_data(G_OBJECT(doc), "liferea-base-uri");
 
-	if(NULL != strstr(url, "file://"))
+	/* source document in local filesystem? */
+	isLocalDoc = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(doc), "localDocument"));
+
+	if(NULL != strstr(url, "file://") || isLocalDoc)
 		absURL = g_strdup(url + strlen("file://"));
 	else
-		absURL = common_build_url(url, g_object_get_data(G_OBJECT(doc), "liferea-base-uri"));
+		absURL = common_build_url(url, base);
 
 	if(absURL != NULL) {
 		struct request *r;
@@ -191,12 +209,15 @@ static void url_request(HtmlDocument *doc, const gchar *url, HtmlStream *stream,
 		sd->doc = doc;
 		sd->stream = stream;
 
-		r = download_request_new();
+		r = update_request_new((gpointer)stream);
+		r->options = g_new0(struct updateOptions, 1);
+		// FIXME: set no proxy option and HTTP auth properties
+		// of selected node to allow viewing of inline images...
 		r->source = g_strdup(absURL);
 		r->callback = gtkhtml2_url_request_received_cb;
 		r->user_data = sd;
 		r->priority = 1;
-		download_queue(r);
+		update_execute_request(r);
 		html_stream_set_cancel_func (stream, stream_cancel, r);
 		g_free(absURL);
 
@@ -213,32 +234,22 @@ static void on_url(HtmlView *view, const char *url, gpointer user_data) {
 	selectedURL = NULL;
 
 	if(NULL != url) {
-
 		gdk_window_set_cursor(GDK_WINDOW(gtk_widget_get_parent_window(GTK_WIDGET(view))), link_cursor);
-		if(strstr(url, "://")) {
-			selectedURL = g_strdup(url);
-		} else {
-			absURL = common_build_url(url, g_object_get_data(G_OBJECT(HTML_VIEW(view)->document), "liferea-base-uri"));
-			if(absURL != NULL) {
-				selectedURL = g_strdup(absURL);
-				xmlFree(absURL);
-			}
+		absURL = common_build_url(url, g_object_get_data(G_OBJECT(HTML_VIEW(view)->document), "liferea-base-uri"));
+		if(absURL != NULL) {
+			selectedURL = g_strdup(absURL);
+			ui_htmlview_on_url(selectedURL);
+			xmlFree(absURL);
 		}
 	} else {
 		gdk_window_set_cursor(GDK_WINDOW(gtk_widget_get_parent_window(GTK_WIDGET(view))), NULL);
+		ui_mainwindow_set_status_bar("");
 	}
-	
-	ui_mainwindow_set_status_bar("%s", selectedURL?selectedURL:"");
 }
 
 static gboolean request_object (HtmlView *view, GtkWidget *widget, gpointer user_data) {
-	GtkWidget *label;
 
-	label = gtk_label_new(_("The included HTML <object> tag is not supported with GtkHTML2.\nIncluded Plugins might not be displayed."));
-	gtk_widget_show(label);
-	gtk_container_add(GTK_CONTAINER (widget), label);
-
-	return TRUE;
+	return FALSE;
 }
 
 static void kill_old_connections (GtkWidget *scrollpane) {
@@ -258,15 +269,25 @@ static void kill_old_connections (GtkWidget *scrollpane) {
 
 static void link_clicked(HtmlDocument *doc, const gchar *url, gpointer scrollpane) {
 	xmlChar		*absURL;
-	
+	gboolean	safeURL, isLocalDoc;
+
 	absURL = common_build_url(url, g_object_get_data(G_OBJECT(doc), "liferea-base-uri"));
 	if(absURL != NULL) {
-		kill_old_connections(GTK_WIDGET(scrollpane));
-		ui_htmlview_launch_URL(GTK_WIDGET(scrollpane), absURL,
-						   GPOINTER_TO_INT(g_object_get_data(G_OBJECT(scrollpane), "internal_browsing")) ?  UI_HTMLVIEW_LAUNCH_INTERNAL: UI_HTMLVIEW_LAUNCH_DEFAULT);
+		/* source document in local filesystem? */
+		isLocalDoc = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(doc), "localDocument"));
+
+		/* prevent launching local filesystem links */	
+		safeURL = (NULL == strstr(absURL, "file://")) || isLocalDoc;
+		
+		if(safeURL) {
+			kill_old_connections(GTK_WIDGET(scrollpane));
+			ui_htmlview_launch_URL(GTK_WIDGET(scrollpane), absURL,
+			                       GPOINTER_TO_INT(g_object_get_data(G_OBJECT(scrollpane), "internal_browsing")) ?  UI_HTMLVIEW_LAUNCH_INTERNAL: UI_HTMLVIEW_LAUNCH_DEFAULT);
+		}
 		xmlFree(absURL);
 	}
 }
+
 void gtkhtml2_destroyed_cb(GtkObject *scrollpane, gpointer user_data) {
 	kill_old_connections(GTK_WIDGET(scrollpane));
 }
@@ -280,7 +301,7 @@ static void gtkhtml2_title_changed(HtmlDocument *doc, const gchar *new_title, gp
 /* ---------------------------------------------------------------------------- */
 
 /* adds a differences diff to the actual zoom level */
-static void change_zoom_level(GtkWidget *scrollpane, gfloat zoomLevel) {
+static void gtkhtml2_change_zoom_level(GtkWidget *scrollpane, gfloat zoomLevel) {
 	GtkWidget *htmlwidget = gtk_bin_get_child(GTK_BIN(scrollpane));
 	
 	/* Clearing the selection is a workaround to avoid 
@@ -292,7 +313,7 @@ static void change_zoom_level(GtkWidget *scrollpane, gfloat zoomLevel) {
 }
 
 /* returns the currently set zoom level */
-static gfloat get_zoom_level(GtkWidget *scrollpane) {
+static gfloat gtkhtml2_get_zoom_level(GtkWidget *scrollpane) {
 	GtkWidget *htmlwidget = gtk_bin_get_child(GTK_BIN(scrollpane));
 	
 	return html_view_get_magnification(HTML_VIEW(htmlwidget));
@@ -301,11 +322,11 @@ static gfloat get_zoom_level(GtkWidget *scrollpane) {
 /* function to write HTML source given as a UTF-8 string. Note: Originally
    the same doc object was reused over and over. To avoid any problems 
    with this now a new one for each output is created... */
-static void write_html(GtkWidget *scrollpane, const gchar *string, guint length,  const gchar *base, const gchar *contentType) {
+static void gtkhtml2_write_html(GtkWidget *scrollpane, const gchar *string, guint length,  const gchar *base, const gchar *contentType) {
 	
 	GtkWidget *htmlwidget = gtk_bin_get_child(GTK_BIN(scrollpane));
 	HtmlDocument	*doc = HTML_VIEW(htmlwidget)->document;
-
+	
 	/* finalizing older stuff */
 	if(NULL != doc) {
 		kill_old_connections(scrollpane);
@@ -317,7 +338,10 @@ static void write_html(GtkWidget *scrollpane, const gchar *string, guint length,
 	
 	doc = html_document_new();
 	html_view_set_document(HTML_VIEW(htmlwidget), doc);
+
 	g_object_set_data(G_OBJECT(doc), "liferea-base-uri", g_strdup(base));
+	g_object_set_data(G_OBJECT(doc), "localDocument", GINT_TO_POINTER(FALSE));
+	
 	html_document_clear(doc);
 	/* Gtkhtml2 only responds to text/html documents, thus everything else must be converted to HTML in Liferea's code */
 	html_document_open_stream(doc, "text/html");
@@ -349,7 +373,7 @@ static void write_html(GtkWidget *scrollpane, const gchar *string, guint length,
 	
 	html_document_close_stream(doc);
 
-	change_zoom_level(scrollpane, get_zoom_level(scrollpane));	/* to enforce applying of changed zoom levels */
+	gtkhtml2_change_zoom_level(scrollpane, gtkhtml2_get_zoom_level(scrollpane));	/* to enforce applying of changed zoom levels */
 	gtkhtml2_scroll_to_top(scrollpane);
 }
 
@@ -367,7 +391,7 @@ static GtkWidget* gtkhtml2_new(gboolean forceInternalBrowsing) {
 	/* create html widget and pack it into the scrolled window */
 	htmlwidget = html_view_new();
 	gtk_container_add (GTK_CONTAINER (scrollpane), GTK_WIDGET(htmlwidget));
-	write_html(scrollpane, NULL, 0, "file:///", NULL);
+	gtkhtml2_write_html(scrollpane, NULL, 0, "file:///", NULL);
 	
 	g_object_set_data(G_OBJECT(scrollpane), "internal_browsing", GINT_TO_POINTER(forceInternalBrowsing));
 	handler = g_signal_connect(G_OBJECT(htmlwidget), "on_url", G_CALLBACK(on_url), NULL);
@@ -386,8 +410,6 @@ static GtkWidget* gtkhtml2_new(gboolean forceInternalBrowsing) {
 	return scrollpane;
 }
 
-
-
 static void gtkhtml2_init() {
 }
 
@@ -395,6 +417,8 @@ static void gtkhtml2_deinit() {
 }
 
 static void gtkhtml2_html_received(struct request *r) {
+	gboolean	isLocalDoc;
+	
 	/* Remove reference to the request structure */
 	g_object_set_data(G_OBJECT(r->user_data), "html_request", NULL);
 	
@@ -404,24 +428,30 @@ static void gtkhtml2_html_received(struct request *r) {
 		return; /* This should nicely exit.... */
 	}
 	ui_tabs_set_location(GTK_WIDGET(r->user_data), r->source);
-	write_html(GTK_WIDGET(r->user_data), r->data, r->size,  r->source, r->contentType);
+	gtkhtml2_write_html(GTK_WIDGET(r->user_data), r->data, r->size,  r->source, r->contentType);
+
+	/* determine if launched URL is a local one and set the flag to allow following local links */
+	isLocalDoc = (r->source == strstr(r->source, "file://"));
+	g_object_set_data(G_OBJECT(HTML_VIEW(gtk_bin_get_child(GTK_BIN(r->user_data)))->document),
+	                  "localDocument", GINT_TO_POINTER(isLocalDoc));
 }
 
-static void launch_url(GtkWidget *scrollpane, const gchar *url) { 
+static void gtkhtml2_launch_url(GtkWidget *scrollpane, const gchar *url) { 
 	struct request *r;
 	
 	kill_old_connections(scrollpane);
-	
-	r = download_request_new();
+
+	r = update_request_new(NULL);
+	r->options = g_new0(struct updateOptions, 1);
 	r->source = g_strdup(url);
 	r->callback = gtkhtml2_html_received;
 	r->user_data = scrollpane;
 	r->priority = 1;
 	g_object_set_data(G_OBJECT(scrollpane), "html_request", r);
-	download_queue(r);
+	update_execute_request(r);
 }
 
-static gboolean launch_inside_possible(void) { return TRUE; }
+static gboolean gtkhtml2_launch_inside_possible(void) { return TRUE; }
 
 /* -------------------------------------------------------------------- */
 /* other functions... 							*/
@@ -469,20 +499,28 @@ static gboolean gtkhtml2_scroll_pagedown(GtkWidget *scrollpane) {
 	return (new_value > old_value);
 }
 
-
-static htmlviewPluginInfo gtkhtml2Info = {
-	HTMLVIEW_API_VERSION,
-	"GtkHTML2",
-	gtkhtml2_init,
-	gtkhtml2_deinit,
-	gtkhtml2_new,
-	write_html,
-	launch_url,
-	launch_inside_possible,
-	get_zoom_level,
-	change_zoom_level,
-	gtkhtml2_scroll_pagedown,
-	/* setProxy = */ NULL
+static struct htmlviewPlugin gtkhtml2Info = {
+	.api_version	= HTMLVIEW_PLUGIN_API_VERSION,
+	.name		= "GtkHTML2",
+	.plugin_init	= gtkhtml2_init,
+	.plugin_deinit	= gtkhtml2_deinit,
+	.create		= gtkhtml2_new,
+	.write		= gtkhtml2_write_html,
+	.launch		= gtkhtml2_launch_url,
+	.launchInsidePossible = gtkhtml2_launch_inside_possible,
+	.zoomLevelGet	= gtkhtml2_get_zoom_level,
+	.zoomLevelSet	= gtkhtml2_change_zoom_level,
+	.scrollPagedown	= gtkhtml2_scroll_pagedown,
+	.setProxy	= NULL,
+	.setOffLine	= NULL
 };
 
+static struct plugin pi = {
+	PLUGIN_API_VERSION,
+	"GtkHTML2 Rendering Plugin",
+	PLUGIN_TYPE_HTML_RENDERER,
+	&gtkhtml2Info
+};
+
+DECLARE_PLUGIN(pi);
 DECLARE_HTMLVIEW_PLUGIN(gtkhtml2Info);
