@@ -1,8 +1,9 @@
 /**
  * @file main.c Liferea main program
  *
- * Copyright (C) 2003 Lars Lindner <lars.lindner@gmx.net>
- * 
+ * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
+ *  
  * Some code like the command line handling was inspired by 
  *
  * Pan - A Newsreader for Gtk+
@@ -36,30 +37,29 @@
 #include <gtk/gtk.h>
 #include <locale.h> /* For setlocale */
 
-#include <sys/types.h> /* Three includes for open(2). My BSD manual says */
-#include <sys/stat.h>  /* to include only <sys/file.h>. I wonder if this */
-#include <fcntl.h>     /* will break any systems. */
-
-#include <sys/types.h> /* For getpid(2) */
-
-#include <unistd.h> /* For gethostname(), readlink(2) and symlink(2) */
 #include <string.h>
-#include <errno.h>
 #include <string.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <unistd.h>
 #include "interface.h"
 #include "support.h"
 #include "callbacks.h"
 #include "feed.h"
+#include "vfolder.h"
 #include "conf.h"
 #include "common.h"
-#include "ui_htmlview.h"
+#include "social.h"
 #include "update.h"
 #include "debug.h"
-#include "ui_mainwindow.h"
-#include "metadata.h"
-#include "ui_session.h"
+#include "ui/ui_mainwindow.h"
+#include "ui/ui_htmlview.h"
+#include "ui/ui_session.h"
+#include "scripting/script.h"
+
+#include "bacon-message-connection.h"
+
+static BaconMessageConnection *bacon_connection = NULL;
 
 gboolean lifereaStarted = FALSE;
 
@@ -74,112 +74,69 @@ static void show_help(void) {
 	g_string_append_printf(str, "%s\n", _("                   Start Liferea with its main window in STATE."));
 	g_string_append_printf(str, "%s\n", _("                   STATE may be `shown', `iconified', or `hidden'"));
 	g_string_append_c(str, '\n');
+	g_string_append_printf(str, "%s\n", _("  --debug-all      Print debugging messages of all types"));
 	g_string_append_printf(str, "%s\n", _("  --debug-cache    Print debugging messages for the cache handling"));
 	g_string_append_printf(str, "%s\n", _("  --debug-conf     Print debugging messages of the configuration handling"));
-	g_string_append_printf(str, "%s\n", _("  --debug-update   Print debugging messages of the feed update processing"));
-	g_string_append_printf(str, "%s\n", _("  --debug-parsing  Print debugging messages of all parsing functions"));
 	g_string_append_printf(str, "%s\n", _("  --debug-gui      Print debugging messages of all GUI functions"));
+	g_string_append_printf(str, "%s\n", _("  --debug-html     Enable HTML debugging (saving to ~/.liferea_1.2/output.xhtml)"));
+	g_string_append_printf(str, "%s\n", _("  --debug-net      Print debugging messages for all network access"));	
+	g_string_append_printf(str, "%s\n", _("  --debug-parsing  Print debugging messages of all parsing functions"));
+	g_string_append_printf(str, "%s\n", _("  --debug-plugins  Print debugging messages when loading plugins"));
 	g_string_append_printf(str, "%s\n", _("  --debug-trace    Print debugging messages when entering/leaving functions"));
-	g_string_append_printf(str, "%s\n", _("  --debug-all      Print debugging messages of all types"));
+	g_string_append_printf(str, "%s\n", _("  --debug-update   Print debugging messages of the feed update processing"));
 	g_string_append_printf(str, "%s\n", _("  --debug-verbose  Print verbose debugging messages"));
 
 	g_string_append_c(str, '\n');
 	g_print("%s", str->str);
 	g_string_free(str, TRUE);
 }
-/**
- * Tries to create a lock file for Liferea.
- *
- * @returns -1 if the lock failed and is locked by someone else. -2
- * for general failures. -3 if there was a stale lockfile. Some
- * non-negative number means success.
- */
 
-static gboolean main_lock() {
-	gchar *filename, *filename2;
-	gchar hostname[256];
-	gint fd;
-	int retval, len;
-	pid_t pid;
-	gchar tmp[300], *host, *pidstr;
+/* bacon message callback */
+static void on_bacon_message_received(const char *message, gpointer data) {
+
+	debug1(DEBUG_GUI, "bacon message received >>>%s<<<", message);
 	
-	if (gethostname(hostname, 256) == -1)
-		return -2; /* Skip locking if this happens, which it should not.... */
-	hostname[255] = '\0';
-	filename = g_strdup_printf("%s" G_DIR_SEPARATOR_S "lock-%s.%d", common_get_cache_path(), hostname, getpid());
-	retval = fd = open(filename, O_CREAT|O_EXCL, S_IRUSR | S_IWUSR);
-	if (fd == -1) {
-		g_free(filename);
-		return -2;
+	/* Currently we only know a single simple command "raise"
+	   which tells the program to raise the window because
+	   another instance was requested which is not supported */
+	   
+	if(g_str_equal(message, "raise")) {
+		debug0(DEBUG_GUI, "-> raise window requested");
+		gtk_window_present(GTK_WINDOW(mainwindow));
+	} else {
+		g_warning("Received unknown bacon command: >>>%s<<<", message);
 	}
-	
-	filename2 = g_strdup_printf("%s" G_DIR_SEPARATOR_S "lock", common_get_cache_path());
-	if (-1 == symlink(filename, filename2)) {
-		if (errno != EEXIST) {
-			retval = -2;
-			goto main_lock_out;
-		}
-		memset(tmp, 0, 300);
-		if ((len = readlink(filename2, tmp, 299)) == -1) {
-			retval = -2; /* Unreadable link, or not a link */
-			goto main_lock_out;
-		}
-		host = strrchr(tmp,'/');
-		if (host == NULL) {
-			retval = -3;
-			goto main_lock_out;
-		}
-		host = strstr(host, "lock-");
-		if (host == NULL) { /* Invalid lock file */
-			retval = -3;
-			goto main_lock_out;
-		}
-		host += strlen("lock-");
-		pidstr = strrchr(host,'.');
-		if (pidstr == NULL) { /* Invalid lock file*/
-			retval = -3;
-			goto main_lock_out;
-		}
-		/* Correct lockfile format */
-		*pidstr = '\0';
-		pidstr++;
-		
-		if (!strcmp(hostname, host)) {
-			pid = atoi(pidstr); /* get PID */
-			if (kill(pid, 0) == 0 || errno != ESRCH)
-				retval = -1;
-			else
-				retval = -3;
-		} else
-			retval = -1;
-	}
- main_lock_out:
-	if (retval == -3) { /* Stale lockfile */
-		fprintf(stderr,"A stale lockfile has been found, and was deleted.\n");
-		unlink(filename2);
-		symlink(filename, filename2); /* Hopefully this will work. If not, screw it. */
-	}
-	close(fd);
-	unlink(filename);
-	g_free(filename);
-	g_free(filename2);
-	
-	return retval;
 }
 
-static void main_unlock() {
-	gchar *filename;
-	
-	filename = g_strdup_printf("%s" G_DIR_SEPARATOR_S "lock", common_get_cache_path());
-	unlink(filename);
-	g_free(filename);
+static void fatal_signal_handler(int sig) {
+	sigset_t sigset;
+
+	sigemptyset(&sigset);
+	sigprocmask(SIG_SETMASK, &sigset, NULL);
+
+	g_print("\nLiferea did receive signal %d (%s).\n", sig, g_strsignal(sig));
+
+	if(debug_level) {
+		g_print("You have propably triggered a program bug. I will now try to \n");
+		g_print("create a backtrace which you can attach to any support requests.\n\n");
+		g_on_error_stack_trace(PACKAGE);
+	}
+
+	_exit(1);
+}
+
+static void signal_handler(int sig) {
+
+	on_quit(NULL, NULL, NULL);
+
+	signal(sig, SIG_DFL);
+	raise(sig);
 }
 
 int main(int argc, char *argv[]) {	
 	gulong		debug_flags = 0;
 	const char 	*arg;
 	gint		i;
-	GtkWidget	*dialog;
 	int mainwindowState = MAINWINDOW_SHOWN;
 #ifdef USE_SM
 	gchar *opt_session_arg = NULL;
@@ -191,15 +148,43 @@ int main(int argc, char *argv[]) {
 	textdomain(GETTEXT_PACKAGE);
 	setlocale(LC_ALL, "");
 #endif
-
+	
 	g_set_prgname("liferea");
 	gtk_set_locale();
 	g_thread_init(NULL);
 #ifdef USE_DBUS
 	dbus_g_thread_init();
+#endif
+#ifdef USE_NM
+	update_nm_initialize();
 #endif
 	gtk_init(&argc, &argv);
 	
+	bacon_connection = bacon_message_connection_new("liferea");
+	if(bacon_connection)	{
+		if(!bacon_message_connection_get_is_server(bacon_connection)) {
+			g_warning(_("Liferea seems to be running already!"));
+			
+		  	debug0(DEBUG_VERBOSE, "Startup as bacon client...");
+			bacon_message_connection_send(bacon_connection, "raise");
+			bacon_message_connection_free(bacon_connection);
+			
+			gdk_notify_startup_complete();
+			exit(0);
+		} else {
+		  	debug0(DEBUG_VERBOSE, "Startup as bacon server...");
+			bacon_message_connection_set_callback(bacon_connection,
+							      on_bacon_message_received,
+							      NULL);
+		}
+	} else {
+		g_warning("Cannot create IPC connection for Liferea!");
+	}
+	
+	/* GTK theme support */
+	g_set_application_name(_("Liferea"));
+	gtk_window_set_default_icon_name("liferea");
+	
 	/* parse arguments  */
 	debug_flags = 0;
 	for(i = 1; i < argc; ++i) {
@@ -215,10 +200,16 @@ int main(int argc, char *argv[]) {
 			debug_flags |= DEBUG_PARSING;
 		else if(!strcmp(arg, "--debug-gui"))
 			debug_flags |= DEBUG_GUI;
+		else if(!strcmp(arg, "--debug-html"))
+			debug_flags |= DEBUG_HTML;
+		else if(!strcmp(arg, "--debug-plugins"))
+			debug_flags |= DEBUG_PLUGINS;
+		else if(!strcmp(arg, "--debug-net"))
+			debug_flags |= DEBUG_NET;
 		else if(!strcmp(arg, "--debug-trace"))
 			debug_flags |= DEBUG_TRACE;
 		else if(!strcmp(arg, "--debug-all"))
-			debug_flags |= DEBUG_TRACE|DEBUG_CACHE|DEBUG_CONF|DEBUG_UPDATE|DEBUG_PARSING|DEBUG_GUI;
+			debug_flags |= DEBUG_TRACE|DEBUG_CACHE|DEBUG_CONF|DEBUG_UPDATE|DEBUG_PARSING|DEBUG_GUI|DEBUG_PLUGINS|DEBUG_NET;
 		else if(!strcmp(arg, "--debug-verbose"))
 			debug_flags |= DEBUG_VERBOSE;		
 		else if(!strcmp(arg, "--version") || !strcmp(arg, "-v")) {
@@ -258,75 +249,61 @@ int main(int argc, char *argv[]) {
 
 	add_pixmap_directory(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "pixmaps");
 
-	if(main_lock() == -1) {
-		dialog = gtk_message_dialog_new(GTK_WINDOW(mainwindow),
-								  0,
-								  GTK_MESSAGE_ERROR,
-								  GTK_BUTTONS_OK,
-								  _("Another copy of Liferea was found to be running. Please use it instead. "
-								  "If there is no other copy of Liferea running, please delete the "
-								  "\"~/.liferea/lock\" lock file."));
-		gtk_dialog_run(GTK_DIALOG (dialog));
-		gtk_widget_destroy(dialog);
+	/* order is important! */
+	conf_init();			/* initialize gconf */
+	update_init();			/* initialize the download subsystem */
+	plugin_mgmt_init();		/* get list of plugins and initialize them */
+	ui_htmlview_init();		/* setup HTML widgets */
+	feed_init();			/* register feed types */
+	vfolder_init();			/* register vfolder rules */
+	conf_load();			/* load global feed settings */
+	script_init();			/* setup scripting if supported */
+	social_init();			/* initialized social bookmarking */
+	ui_mainwindow_init(mainwindowState);	/* setup mainwindow and initialize gconf configured GUI behaviour */
 
-	} else {
-		/* order is important! */
-		conf_init();			/* initialize gconf */
-		ui_htmlview_init();		/* setup HTML widgets */
-		download_init();		/* Initialize the download subsystem */
-		metadata_init();
-		feed_init();			/* register feed types */
-		conf_load();			/* load global feed settings */
-		ui_init(mainwindowState);	/* setup mainwindow and initialize gconf configured GUI behaviour */
 #ifdef USE_SM
-		/* This must be after feedlist reading because some session
-		   managers will tell Liferea to exit if Liferea does not
-		   respond to SM requests within a minute or two. This starts
-		   the main loop soon after opening the SM connection. */
-		session_init(BIN_DIR G_DIR_SEPARATOR_S "liferea", opt_session_arg);
-		session_set_cmd(NULL, mainwindowState);
+	/* This must be after feedlist reading because some session
+	   managers will tell Liferea to exit if Liferea does not
+	   respond to SM requests within a minute or two. This starts
+	   the main loop soon after opening the SM connection. */
+	session_init(BIN_DIR G_DIR_SEPARATOR_S "liferea", opt_session_arg);
+	session_set_cmd(NULL, mainwindowState);
 #endif
+	signal(SIGTERM, signal_handler);
+	signal(SIGINT, signal_handler);
+	signal(SIGHUP, signal_handler);
+
+#ifndef G_OS_WIN32
+	signal(SIGBUS, fatal_signal_handler);
+	signal(SIGSEGV, fatal_signal_handler);
+#endif
+
+	/* Note: we explicitely do not use the gdk_thread_*
+	   locking in Liferea because it freezes the program
+	   when running Flash applets in gtkmozembed */
+
+	lifereaStarted = TRUE;
+	gtk_main();
 		
-		/* Note: we explicitely do not use the gdk_thread_*
-		   locking in Liferea because it freezes the program
-		   when running Flash applets in gtkmozembed */
-
-		lifereaStarted = TRUE;
-		gtk_main();
-	}
-
+	bacon_message_connection_free(bacon_connection);
 	return 0;
 }
 
 gboolean on_quit(GtkWidget *widget, GdkEvent *event, gpointer user_data) {
-	GtkWidget	*pane;
-	gint x, y;
 
 	debug_enter("on_quit");
 
+	script_run_for_hook(SCRIPT_HOOK_SHUTDOWN);
+
 	ui_mainwindow_save_position();
 	gtk_widget_hide(mainwindow);
 
-	ui_itemlist_clear();	/* to free all item list related data */
-	conf_feedlist_save();	/* should save feedlist and folder states */
+	ui_feedlist_select(NULL);	/* should unload/save selected node and items */
+	feedlist_save();		/* should save feedlist and folder states */
 
 	/* should save all feeds still in memory */	
-	ui_feedlist_do_for_all(NULL, ACTION_FILTER_FEED | ACTION_FILTER_DIRECTORY, feedlist_unload_feed);
+	feedlist_foreach(node_unload);
 	
-	/* save pane proportions */
-	if(NULL != (pane = lookup_widget(mainwindow, "leftpane"))) {
-		x = gtk_paned_get_position(GTK_PANED(pane));
-		setNumericConfValue(LAST_VPANE_POS, x);
-	}
-	
-	if(NULL != (pane = lookup_widget(mainwindow, "rightpane"))) {
-		y = gtk_paned_get_position(GTK_PANED(pane));
-		setNumericConfValue(LAST_HPANE_POS, y);
-	}
-	
-	/* save itemlist properties */
-	setNumericConfValue(LAST_ZOOMLEVEL, (gint)(100.*ui_htmlview_get_zoom(ui_mainwindow_get_active_htmlview())));
-
 	gtk_widget_destroy(mainwindow);
 	ui_htmlview_deinit();
 	
@@ -334,10 +311,9 @@ gboolean on_quit(GtkWidget *widget, GdkEvent *event, gpointer user_data) {
 	/* unplug */
 	session_end();
 #endif
+	
 	gtk_main_quit();
-	main_unlock();
 	
 	debug_exit("on_quit");
-	
 	return FALSE;
 }
