@@ -23,9 +23,10 @@
 
 #include "../support.h"
 #include "../callbacks.h"
+#include "../debug.h"
 
 /* we redefine some SnowNews functions */
-#define UIStatus(a, b)	ui_mainwindow_set_status_bar("%s", a)
+#define UIStatus(a, b)		
 #define MainQuit(str, errno)	g_error(str);
 #define	getch()			0
 
