@@ -24,7 +24,10 @@
 #define NET_SUPPORT_H
 
 #include "compat.h"
-
+#include "netio.h"
 int NetSupportAuth (struct feed_request * cur_ptr, char * authdata, char * url, char * netbuf);
+char * ConstructBasicAuth (char * username, char * password);
+int checkValidHTTPHeader (const unsigned char * header, int size);
+int checkValidHTTPURL (const unsigned char * url);
 
 #endif
